import platform
import sys
from importlib import metadata
from fleting.cli.console.console import console

BANNER = r"""
 ______ _      _   _             
|  ____| |    | | (_)            
| |__  | | ___| |_ _ _ __   __ _ 
|  __| | |/ _ \ __| | '_ \ / _` |
| |    | |  __/ |_| | | | | (_| |
|_|    |_|\___|\__|_|_| |_|\__, |
                            __/ |
                           |___/
"""

def _get_version(pkg_name: str):
    try:
        return metadata.version(pkg_name)
    except metadata.PackageNotFoundError:
        return "not installed"

def handle_info():
    python_version = sys.version.split()[0]
    system = f"{platform.system()} {platform.release()}"

    flet_version = _get_version("flet")
    fleting_version = _get_version("fleting")

    console.print(BANNER)
    console.print("🚀 Fleting Framework\n", style="header")

    console.print("📦 Environment\n", style="subtitle")
    console.print(f"🧠 Python        : {python_version}", style="label")
    console.print(f"🖥️  System      : {system}", style="label")
    console.print(f"🧩 Flet          : {flet_version}", style="label")
    console.print(f"🚀 Fleting       : {fleting_version}", style="label")

    console.print("\n📚 Installed libraries:", style="subtitle")
    for dist in sorted(metadata.distributions(), key=lambda d: d.metadata["Name"].lower()):
        name = dist.metadata["Name"]
        version = dist.version
        console.print(f"  - {name}=={version}", style="muted")

    console.print("\n✅ Ready-to-use environment.\n", style="success")



from rich.theme import Theme

THEMES = Theme({
    # Main states - matching current CLI emoji style
    "success": "green1 bold",
    "error": "red1 bold", 
    "warning": "yellow1 bold",
    "info": "cyan bold",
    
    # Soft variants
    "success.dim": "green3",
    "error.dim": "red3", 
    "warning.dim": "yellow3",
    "info.dim": "cyan3",
    
    # UX / hints
    "suggestion": "magenta italic",
    "hint": "bright_black italic",
    "muted": "bright_black",
    
    # Structure
    "title": "bold white underline",
    "subtitle": "bold white",
    "label": "bold blue",
    "value": "white",
    
    # Debug / dev stuff
    "debug": "dim cyan",
    "trace": "dim bright_black",
    
    # Fancy extras
    "highlight": "black on yellow",
    "important": "bold white on red",
    
    # CLI specific themes
    "command": "bold green",
    "description": "white",
    "header": "bold cyan",
    "table.border": "blue",
    "table.header": "bold blue"
})



from pathlib import Path
import importlib.util
import sys
from typing import Dict, List, Set
from fleting.cli.console import (
    console, 
    print_simple_table,     
    print_pages_table,       
    print_routes_table,      
    create_table
)

# ----------------------
# Utils
# ----------------------
def get_project_root() -> Path:
    return Path.cwd()

def is_fleting_project(path: Path) -> bool:
    return (path / "main.py").exists() and (path / "configs").exists()

def print_table_2(headers, rows):
    col_sizes = [len(h) for h in headers]

    for row in rows:
        for i, cell in enumerate(row):
            col_sizes[i] = max(col_sizes[i], len(str(cell)))

    def fmt(row):
        return " | ".join(str(cell).ljust(col_sizes[i]) for i, cell in enumerate(row))

    console.print(fmt(headers))
    console.print("-+-".join("-" * s for s in col_sizes))

    for row in rows:
        console.print(fmt(row))

def print_table(headers: List[str], rows: List[List[str]], title: str = ""):
    """Función de compatibilidad para reemplazar la antigua print_table"""
    table = create_table(title, [(h, "bold cyan", 20) for h in headers])
    
    for row in rows:
        styled_row = []
        for i, cell in enumerate(row):
            if headers[i].lower() in ["model", "controller", "view"]:
                if cell == "✔":
                    styled_cell = "[green]✔[/green]"
                else:
                    styled_cell = "[dim red]—[/dim red]"
            elif headers[i].lower() == "route":
                styled_cell = f"[yellow]{cell}[/yellow]"
            else:
                styled_cell = f"[cyan]{cell}[/cyan]"
            styled_row.append(styled_cell)
        
        table.add_row(*styled_row)
    
    console.print()
    console.print(table)
# ----------------------
# Handlers
# ----------------------
def handle_list(args):
    root = get_project_root()

    if not is_fleting_project(root):
        console.print("❌ This directory is not a Fleting project.", style="error")
        return

    if not args:
        console.print("[bold yellow]Usage:[/bold yellow] fleting list <pages|controllers|views|models|routes>")
        console.print("\n[yellow]Available options:[/yellow]")
        options_table = console.create_table(columns=[("Option", "bold green", 15), ("Description", "cyan", 40)])
        options_table.add_row("pages", "List all pages with MVC components")
        options_table.add_row("controllers", "List all controllers")
        options_table.add_row("views", "List all views")
        options_table.add_row("models", "List all models")
        options_table.add_row("routes", "List all routes")
        console.print(options_table)
        return
        # console.print("Use: fleting list <pages|controllers|views|models|routes>", style="suggestion")
        # return

    kind = args[0]

    if kind == "controllers":
        list_simple(root / "controllers", "_controller.py", "Controller")
    elif kind == "views":
        list_simple(root / "views" / "pages", "_view.py", "View")
    elif kind == "models":
        list_simple(root / "models", "_model.py", "Model")
    elif kind == "routes":
        list_routes(root)
    elif kind == "pages":
        list_pages(root)
    else:
        console.print(f"Unknown list type: {kind}", style="warning")
        console.print(f"[dim]Available options: pages, controllers, views, models, routes[/dim]")

# ----------------------
# Simple lists
# ----------------------
def list_simple(path: Path, suffix: str, title: str):
    rows = []

    if not path.exists():
        console.print(f"No {title.lower()} found.", style="warning")
        return

    for file in sorted(path.glob(f"*{suffix}")):
        name = file.stem.replace(suffix.replace(".py", ""), "")
        file_size = file.stat().st_size
        rows.append([name, file.name, f"{file_size:,} bytes"])

    print_simple_table(
        headers=[
            ("Name", "bold green", 20),
            ("File", "cyan", 25),
            ("Size", "dim", 15)
        ],
        rows=rows,
        title=title
    )

# ----------------------
# Pages (MVC + route)
# ----------------------
def list_pages(root: Path):
    rows = []

    models = {f.stem.replace("_model", "") for f in (root / "models").glob("*_model.py")}
    controllers = {f.stem.replace("_controller", "") for f in (root / "controllers").glob("*_controller.py")}
    views = {f.stem.replace("_view", "") for f in (root / "views" / "pages").glob("*_view.py")}

    routes = load_routes(root)

    all_pages = sorted(models | controllers | views | routes.keys())

    for name in all_pages:
        rows.append([
            name,
            "✔" if name in models else "—",
            "✔" if name in controllers else "—",
            "✔" if name in views else "—",
            f"/{name}" if name in routes else "—",
        ])

    print_table(
        ["Page", "Model", "Controller", "View", "Route"],
        rows
    )

# ----------------------
# Routes
# ----------------------
def list_routes(root: Path):
    routes = load_routes(root)

    if not routes:
        console.print("No routes found.", style="warning")
        return

    rows = []
    for path, view in sorted(routes.items()):
        # Extraer nombre del controlador de la view string
        view_name = view.split('.')[-1] if '.' in view else view
        rows.append([f"/{path}", view_name])

    print_routes_table(
        headers=[
            ("Route", "bold green", 25),
            ("View", "cyan", 30)
        ],
        rows=rows
    )

def load_routes(root: Path) -> Dict[str, str]:
    """charge route from routes.py"""
    routes_file = root / "configs" / "routes.py"
    
    if not routes_file.exists():
        return {}

    try:
        spec = importlib.util.spec_from_file_location("routes", routes_file)
        routes_module = importlib.util.module_from_spec(spec)
        sys.modules["routes"] = routes_module
        spec.loader.exec_module(routes_module)
        
        routes_dict = {}
        for r in routes_module.ROUTES:
            path = r["path"].replace("/", "")
            routes_dict[path] = r["view"]
        return routes_dict
    except Exception as e:
        console.print(f"Could not load routes: {e}", style="warning")
        return {}


from pathlib import Path
from fleting.cli.templates.database import db_init, db_migrate, db_seed, make_migration, db_rollback, db_status
from fleting.cli.commands.pull import handle_pull
from fleting.cli.console.console import console

def is_fleting_project(path: Path) -> bool:
    return (path / ".fleting").exists()

def find_project_root(start=None) -> Path | None:
    if start is None:
        start = Path.cwd()
    elif isinstance(start, str):
        start = Path(start)

    start = start.resolve()

    if is_fleting_project(start):
        return start

    for parent in start.parents:
        if is_fleting_project(parent):
            return parent

    return None

def activate_project(root):
    from pathlib import Path
    import sys

    if isinstance(root, str):
        root = Path(root)

    root = root.resolve()

    if str(root) not in sys.path:
        sys.path.insert(0, str(root))

def get_project_root():
    root = find_project_root()
    if not root:
        console.print("❌ This directory is not a Fleting project.", style="error")
        console.print("👉 Go to the project root or a parent directory.", style="suggestion")
        return

    activate_project(root)
    return root

def handle_db(args):
    if not args:
        console.print("Use: fleting db <init|migrate|seed|make|rollback|status>", style="suggestion")
        return

    root = get_project_root()
    if not root:
        return

    cmd = args[0]

    if cmd == "init":
        db_init(root)
    elif cmd == "migrate":
        db_migrate(root)
    elif cmd == "seed":
        db_seed(root)
    elif cmd == "make":
        if len(args) < 2:
            console.print("Use: fleting db make <name>\n", style="suggestion")
            return
        make_migration(root, args[1])
    elif cmd == "rollback":
        db_rollback(root)
    elif cmd == "status":
        db_status(root)
    elif cmd == "model":
        handle_pull(args[1:])
    else:
        console.print(f"Unknown db command: {cmd}", style="warning")





# cli/console/__init__.py
from .console import (
    console,
    Console,
    Table,
    Panel,
    Columns,
    Text,
    box,
    Rule,
    Syntax,
    Markdown,
    Progress,
    SpinnerColumn,
    TextColumn,
    Prompt,
    Confirm,
    Tree,
    create_table,
    print_pages_table,
    print_routes_table,
    print_simple_table,
)

__all__ = [
    'console',
    'Console',
    'Table',
    'Panel',
    'Columns',
    'Text',
    'box',
    'Rule',
    'Syntax',
    'Markdown',
    'Progress',
    'SpinnerColumn',
    'TextColumn',
    'Prompt',
    'Confirm',
    'Tree',
    'create_table',
    'print_pages_table',
    'print_routes_table',
    'print_simple_table',
]


import code
from pathlib import Path
from fleting.cli.console.console import console
import importlib
import inspect
import sys
from rich.table import Table

class FletingPrompt:
    def __init__(self):
        self.counter = 0

    def __str__(self):
        self.counter += 1
        YELLOW = "\033[33m"
        GREEN = "\033[32m"
        RESET = "\033[0m"

        return f"{YELLOW}[{self.counter}] Fleting{GREEN} ❯_ {RESET}"

sys.ps1 = FletingPrompt()

def is_fleting_project(path: Path) -> bool:
    return (path / ".fleting").exists()

def find_project_root(start=None) -> Path | None:
    if start is None:
        start = Path.cwd()
    elif isinstance(start, str):
        start = Path(start)

    start = start.resolve()

    if is_fleting_project(start):
        return start

    for parent in start.parents:
        if is_fleting_project(parent):
            return parent

    return None

def activate_project(root):
    from pathlib import Path
    import sys

    if isinstance(root, str):
        root = Path(root)

    root = root.resolve()

    if str(root) not in sys.path:
        sys.path.insert(0, str(root))

def get_project_root():
    root = find_project_root()
    if not root:
        console.print("❌ This directory is not a Fleting project.", style="error")
        console.print("👉 Go to the project root or a parent directory.", style="suggestion")
        return

    activate_project(root)
    return root

def find_database(project_root: Path) -> Path | None:
    data_dir = project_root / "data"

    if not data_dir.exists():
        return None

    db_files = list(data_dir.glob("*.db"))
    return db_files[0] if db_files else None

def load_models(project_root: Path):
    models_dir = project_root / "models"
    loaded = {}

    if not models_dir.exists():
        return loaded

    for file in models_dir.glob("*.py"):
        if file.name.startswith("_"):
            continue

        module_name = f"models.{file.stem}"
        module = importlib.import_module(module_name)

        for name, obj in inspect.getmembers(module, inspect.isclass):
            if obj.__module__ == module_name:
                loaded[name] = obj

    return loaded

def handle_shell():
    
    project_root = get_project_root()

    if not project_root:
        console.print("[error]No Fleting projects found.[/error]")
        return
    
    db_path = find_database(project_root)
    if not db_path:
        console.print(
            "[error]❌ No local databases found.[/error]\n"
            "[suggestion]👉 Execute: fleting db init & fleting migrate [/suggestion]"
        )
        return
    
    # =========================
    # Database
    # =========================
    from configs.database import DATABASE
    engine = DATABASE.get("ENGINE", "sqlite").lower()
    conn = None
    cursor = None
    
    if engine == "sqlite":
        console.print("[success]🟢 SQLite detected[/success]")
        try:
            from core.database import get_connection
            import sqlite3

            conn = get_connection()
            conn.row_factory = sqlite3.Row
            cursor = conn.cursor()
        except Exception as e:
            console.print(f"[error]Error obtaining connection: {e}[/error]")
            return
    else:
        console.print(
            f"[error]❌ Engine '{engine}' not supported by the shell.[/error]"
        )
        return
    
    # =========================
    # Helpers
    # =========================
    def tables():
        cursor.execute(
            "SELECT name FROM sqlite_master WHERE type='table';"
        )
        return [row["name"] for row in cursor.fetchall()]

    def query(sql: str):
        cursor.execute(sql)
        rows = cursor.fetchall()
        return [dict(row) for row in rows]
    
    def table_exists(cursor, table_name: str) -> bool:
        cursor.execute(
            "SELECT name FROM sqlite_master WHERE type='table' AND name=?",
            (table_name,)
        )
        return cursor.fetchone() is not None
    
    def table_view(table_name: str):
        if not table_exists(cursor, table_name):
            console.print("[error]Table not found[/error]")
            return
        
        cursor.execute(f"SELECT * FROM {table_name}")
        rows = cursor.fetchall()

        if not rows:
            console.print("[warning]Table exists but has no records[/warning]")
            return

        table = Table(title=f"📊 {table_name}")

        for col in rows[0].keys():
            table.add_column(col, style="cyan", no_wrap=True)

        for row in rows:
            table.add_row(*[str(v) for v in row])

        console.print(table)
    
    # =========================
    # Load models (REAL ORM)
    # =========================
    all_models_map = load_models(project_root)

    def models():
        return list(all_models_map.keys())

    banner = f"""
[bold cyan]Fleting Console[/bold cyan]
Project: [green]{project_root.name}[/green]
Database: [yellow]{db_path.name}[/yellow]

Available helpers:
• db/conn → SQLite connection
• cursor → SQLite cursor
• tables() → list tables
• query(sql) → execute SELECT
• models() → list models
• <Model>.all() → list records
• table("table_name") → execute SELECT FROM

Use quit() or Ctrl-Z plus Return to exit
"""

    console.print(banner)

    context = {
        "project_root": project_root,
        "Path": Path,
         # DB
        "db": conn,
        "conn": conn,
        "cursor": cursor,

        # Helpers
        "models": models,
        "tables": tables,
        "query": query,
        "table": table_view,
    }

    context.update(all_models_map)

    try:
        code.interact(banner="", local=context)
    except SystemExit:
        console.print("\n[info]👋 Leaving the Fleting Shell[/info]")




import sys
from fleting.cli.commands.init import handle_init
from fleting.cli.commands.run import handle_run
from fleting.cli.commands.info import handle_info
from fleting.cli.commands.create import handle_create
from fleting.cli.commands.delete import handle_delete
from fleting.cli.commands.list import handle_list
from fleting.cli.commands.db import handle_db
from fleting.cli.commands.shell import handle_shell
from fleting.cli.commands.pull import handle_pull

from fleting.cli.console.console import console
from fleting.cli.console import Table, Panel, Text, box, Rule
# import traceback

def print_help():
    console.clear()
    
    # Título principal
    console.print()
    console.rule("[bold cyan]🚀 Fleting CLI[/bold cyan]", style="cyan")
    console.print()
    
    # Panel de uso
    console.print(Panel.fit(
        "[bold green]fleting <command> [options][/bold green]",
        title="[yellow]📌 Usage[/yellow]",
        border_style="yellow",
        padding=(1, 2)
    ))
    console.print()
    
    table = Table(
        show_header=True,
        header_style="bold magenta",
        box=box.ROUNDED,
        border_style="blue",
        padding=(0, 1)
    )
    
    table.add_column("Command", style="bold green", width=36)
    table.add_column("Description", style="cyan", width=44)
    
    # Comandos organizados por categorías
    categories = {
        "💼 Project Commands": [
            ("fleting init <app_name>", "Initialize a new Fleting project"),
            ("fleting info", "Show version and system information"),
            ("fleting run", "Run the application"),
        ],
        "➕ Create Commands": [
            ("fleting create page <name>", "Create a page (model + controller + view)"),
            ("fleting create view <name>", "Create a new view"),
            ("fleting create model <name>", "Create a new model"),
            ("fleting create controller <name>", "Create a new controller"),
        ],
        "🗑️  Delete Commands": [
            ("fleting delete page <name>", "Delete an existing page"),
            ("fleting delete view <name>", "Delete a view"),
            ("fleting delete model <name>", "Delete a model"),
            ("fleting delete controller <name>", "Delete a controller"),
        ],
        "📋 List Commands": [
            ("fleting list pages", "List all pages"),
            ("fleting list controllers", "List all controllers"),
            ("fleting list views", "List all views"),
            ("fleting list models", "List all models"),
            ("fleting list routes", "List all routes"),
        ],
        "🗄️  Database Commands": [
            ("fleting db init", "Initialize the database"),
            ("fleting db migrate", "Run database migrations"),
            ("fleting db seed", "Seed the database with initial data"),
            ("fleting db make <name>", "Create a new migration"),
            ("fleting db rollback", "Rollback the last migration"),
            ("fleting db status", "Show current database migration status"),
        ],
        "🗄️  Database / Models": [
            ("fleting db model pull", "Generate models from the database schema"),
            ("fleting db model pull <table>", "Generate model for a specific table"),
            ("fleting db model pull <table> --force", "Overwrite existing model files"),
        ],
        "🧩  Console Command": [
            ("fleting shell", "Open interactive database shell (SQLite only)"),
        ],
    }
    
    for category, commands in categories.items():
        # Añadir separador para categorías
        if list(categories.keys()).index(category) > 0:
            table.add_row("─" * 36, "─" * 44, style="dim")
        
        # Añadir título de categoría
        table.add_row(
            f"[bold yellow]{category}[/bold yellow]",
            "",
            style="yellow"
        )
        
        # Añadir comandos de la categoría
        for cmd, desc in commands:
            table.add_row(f"  {cmd}", desc)
    
    console.print(table)
    console.print()

def main():
    
    args = sys.argv[1:]

    if not args or args[0] in ("-h", "--help"):
        print_help()
        return

    command = args[0]
    
    try:
        if command == "init":
            handle_init(args[1:])
        elif command == "run":
            handle_run()
        elif command == "info":
            handle_info()
        elif command == "create":
            handle_create(args[1:])
        elif command == "delete":
            handle_delete(args[1:])
        elif command == "list":
            handle_list(args[1:])
        elif command == "db":
            handle_db(args[1:])
        elif command == "shell":
            handle_shell()
        elif command == "pull":
            handle_pull(args[1:])
        else:
            print(f"Unknown command: {command}")
            print_help()

    except Exception as e:
         print("Error executing CLI command:", str(e))
         # traceback.print_exc() 

if __name__ == "__main__":
    main()



import importlib.util
from pathlib import Path
import datetime

def db_init(project_root: Path):

    BASE = project_root
    # =========================
    # UTIL
    # =========================
    def create_file(path, content=""):
        path.parent.mkdir(parents=True, exist_ok=True)
        if not path.exists():
            path.write_text(content, encoding="utf-8")
    
    # =========================
    # ESTRUTURA DE PASTAS
    # =========================
    folders = [
        "migrations",
        "seeds",
        "data",
    ]

    for folder in folders:
        (BASE / folder).mkdir(parents=True, exist_ok=True)

    init_file = BASE / "migrations" / "__init__.py"
    if not init_file.exists():
        init_file.write_text("", encoding="utf-8")

    create_file(BASE / "migrations/001_initial.py", """
def up(db):
    db.execute('''
        CREATE TABLE IF NOT EXISTS users (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            username TEXT UNIQUE,
            password TEXT
        )
    ''')

def down(db):
    db.execute('''
        DROP TABLE IF EXISTS users
    ''')               
""")
    print(f"✅ migrations/001_initial.py successfully created")
    
    create_file(BASE / "seeds/initial.py", """
def run(db):
    db.execute('''
        INSERT OR IGNORE INTO users (username, password)
        VALUES ('admin', 'fleting')
    ''')
""")
    print(f"✅ seeds/initial.py successfully created")

    create_file(BASE / "core/migrations.py", """
from pathlib import Path
from core.database import get_connection

MIGRATIONS_TABLE = "_fleting_migrations"

def ensure_migrations_table():
    db = get_connection()
    db.execute(f'''
        CREATE TABLE IF NOT EXISTS {MIGRATIONS_TABLE} (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT UNIQUE
        )
    ''')
    db.commit()

def applied_migrations():
    db = get_connection()
    ensure_migrations_table()
    rows = db.execute(
        f"SELECT name FROM {MIGRATIONS_TABLE} ORDER BY id"
    ).fetchall()
    return [r[0] for r in rows]

def apply_migration(name, up):
    db = get_connection()
    up(db)
    db.execute(
        f"INSERT INTO {MIGRATIONS_TABLE} (name) VALUES (?)",
        (name,)
    )
    db.commit()

def rollback_migration(name, down):
    db = get_connection()
    down(db)
    db.execute(
        f"DELETE FROM {MIGRATIONS_TABLE} WHERE name = ?",
        (name,)
    )
    db.commit()
""")
     
    print("✅ Database structure initialized")

def db_migrate(root: Path):
    from core.migrations import applied_migrations, apply_migration

    migrations_dir = root / "migrations"

    if not migrations_dir.exists():
        print("❌ Run `fleting db init` first.")
        return

    applied = applied_migrations()

    files = sorted(migrations_dir.glob("*.py"))
    files = [f for f in files if f.name != "__init__.py"]

    if not files:
        print("⚠️ No migrations found.")
        return

    for file in files:
        if file.name in applied:
            continue

        spec = importlib.util.spec_from_file_location(file.stem, file)
        mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mod)

        if not hasattr(mod, "up"):
            print(f"❌ Migration {file.name} missing up()")
            continue

        apply_migration(file.name, mod.up)
        print(f"✅ Applied migration {file.name}")

def db_seed(root: Path):
    seeds_dir = root / "seeds"

    if not seeds_dir.exists():
        print("❌ No seeds directory.")
        return

    from core.database import get_connection
    db = get_connection()

    for file in sorted(seeds_dir.glob("*.py")):
        spec = importlib.util.spec_from_file_location(file.stem, file)
        mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mod)

        if hasattr(mod, "run"):
            mod.run(db)
            print(f"🌱 Seed executed: {file.name}")

    db.commit()

def make_migration(root: Path, name: str):
    migrations_dir = root / "migrations"
    migrations_dir.mkdir(exist_ok=True)

    existing = sorted(migrations_dir.glob("*.py"))
    nums = [
        int(f.name.split("_")[0])
        for f in existing
        if f.name[0:3].isdigit()
    ]
    next_num = max(nums, default=0) + 1

    filename = f"{next_num:03d}_{name}.py"
    path = migrations_dir / filename

    template = f'''"""
Migration: {name}
Created at: {datetime.datetime.now().isoformat()}
"""

def up(db):
    # write your SQL here
    pass

def down(db):
    pass
'''

    path.write_text(template, encoding="utf-8")
    print(f"✅ Migration created: {filename}")

def db_rollback(root):
    from core.migrations import applied_migrations, rollback_migration

    migrations_dir = root / "migrations"
    applied = applied_migrations()

    if not applied:
        print("⚠️ No migrations to rollback.")
        return

    last = applied[-1]
    file = migrations_dir / last

    if not file.exists():
        print(f"❌ Migration file not found: {last}")
        return

    spec = importlib.util.spec_from_file_location(file.stem, file)
    mod = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(mod)

    if not hasattr(mod, "down"):
        print(f"❌ Migration {last} has no down()")
        return

    rollback_migration(last, mod.down)
    print(f"↩️ Rolled back migration {last}")

def db_status(root):
    from core.migrations import applied_migrations

    migrations_dir = root / "migrations"

    if not migrations_dir.exists():
        print("❌ No migrations directory. Run `fleting db init` first.")
        return

    applied = applied_migrations()
    files = sorted(
        f.name for f in migrations_dir.glob("*.py")
        if f.name != "__init__.py"
    )

    applied_set = set(applied)
    files_set = set(files)

    pending = sorted(files_set - applied_set)
    missing = sorted(applied_set - files_set)

    print("\n📦 Database status\n")

    if applied:
        print("Applied migrations:")
        for name in applied:
            print(f"  ✔ {name}")
    else:
        print("Applied migrations:")
        print("  (none)")

    print()

    if pending:
        print("Pending migrations:")
        for name in pending:
            print(f"  ⏳ {name}")
    else:
        print("Pending migrations:")
        print("  (none)")

    if missing:
        print("\n⚠️ Inconsistencies detected:")
        for name in missing:
            print(f"  ❌ Applied but file missing: {name}")

    if not pending and not missing:
        print("\n✅ Database is up to date.")



import subprocess
import sys
import shutil
from pathlib import Path
from fleting.cli.console.console import console

def handle_run():
    project_root = Path.cwd()
    app_path = project_root / "main.py"

    if not app_path.exists():
        console.print("❌ main.py not found.", style="error")
        console.print("👉 Execute this command within a Fleting project.", style="suggestion")
        return

    if not shutil.which("flet"):
        console.print("❌ Flet is not installed in the environment.", style="error")
        console.print("👉 pip install flet", style="suggestion")
        return

    console.print("🚀 Starting Fleting application..\n", style="info")

    try:
        subprocess.run(
            ["flet", "run", str(app_path)],
            check=True
        )
    except subprocess.CalledProcessError:
        console.print("❌ Error running the app with Flet", style="error")



from pathlib import Path
import sys
from fleting.cli.helpers.project import get_project_root

from rich.console import Console

console = Console()

def find_database(project_root: Path) -> Path | None:
    data_dir = project_root / "data"

    if not data_dir.exists():
        return None

    db_files = list(data_dir.glob("*.db"))
    return db_files[0] if db_files else None

def get_tables(cursor):
    cursor.execute("""
        SELECT name FROM sqlite_master
        WHERE type='table'
        AND name NOT LIKE 'sqlite_%'
    """)
    return [row[0] for row in cursor.fetchall()]

def get_columns(cursor, table):
    cursor.execute(f"PRAGMA table_info({table})")
    return cursor.fetchall()

def get_foreign_keys(cursor, table: str):
    cursor.execute(f"PRAGMA foreign_key_list({table})")
    return cursor.fetchall()

def map_sqlite_type(sqlite_type: str) -> str:
    """Mapea tipos de SQLite a tipos de Python."""
    stype = sqlite_type.upper()
    if "INT" in stype:
        return "int"
    if "CHAR" in stype or "TEXT" in stype:
        return "str"
    if "FLOAT" in stype or "REAL" in stype or "NUMERIC" in stype:
        return "float"
    if "BLOB" in stype:
        return "bytes"
    return "Any" 

def generate_model(table, columns, foreign_keys, all_tables) -> str:

    class_name = "".join(word.capitalize() for word in table.split('_'))

    lines = [
        "from dataclasses import dataclass",
        "from typing import Any, Optional, ClassVar",
        "from core.base_model import BaseModel",
        "from core.orm import ForeignKey, HasMany",
        "",
        "@dataclass",
        f"class {class_name}Model(BaseModel):",
        f'    table_name: str = "{table}"',
        ""
    ]

    if not columns:
        lines.append("    pass")
    else:
        lines.append("    # columns")
        for col in columns:
            # _, name, col_type, notnull, default, pk = col
            _, name, col_type, *_ = col
            py_type = map_sqlite_type(col_type)
            lines.append(f"    {name}: Optional[{py_type}] = None")
        
        # ForeignKey relations
        if foreign_keys:
            lines.append("")
            lines.append("    # relations (ForeignKey)")

            for fk in foreign_keys:
                ref_table = fk[2]
                local_col = fk[3]
                remote_col = fk[4]

                attr = ref_table.rstrip("s")
                # Relations fora do __init__
                lines.append(
                    f'    {attr} = ForeignKey('
                    f'"{ref_table}", local="{local_col}", remote="{remote_col}")'
                )
        
        # HasMany (inverse relation)
        for other_table, fks in all_tables.items():
            for fk in fks:
                if fk[2] == table:
                    attr = other_table
                    lines.append(
                        f'    {attr} = HasMany('
                        f'"{other_table}", foreign_key="{fk[3]}")'
                    )

    return "\n".join(lines)

def save_model(project_root, table, content, force):
    import pathlib
    root = pathlib.Path(project_root).resolve()
    models_dir = root / "models"
    models_dir.mkdir(parents=True, exist_ok=True)
    
    path = models_dir / f"{table}_model.py"
    class_name = "".join(word.capitalize() for word in table.split('_'))

    default_pattern = f"""@dataclass
class {class_name}Model(BaseModel):
    table_name: str = "{table}"
"""

    if path.exists():
        current_content = path.read_text(encoding="utf-8").strip()
        
        normalized_current = "\n".join([line.strip() for line in current_content.splitlines() if line.strip()])
        normalized_default = "\n".join([line.strip() for line in default_pattern.splitlines() if line.strip()])

        if normalized_current != normalized_default and current_content != "":
            console.print(
                f"[warning]⚠️  Model {table}.py already contains custom logic. Ignored.[/warning]"
            )
            return
        else:
            console.print(f"[info]i Updating base model for: {table}.py[/info]")

    path.write_text(content, encoding="utf-8")
    console.print(f"[success]✔ Model generated in: [bold]{path.absolute()}[/bold][/success]")

def register_db_commands(subparsers):
    db_parser = subparsers.add_parser("db")

    db_sub = db_parser.add_subparsers(dest="db_command")

    model_parser = db_sub.add_parser("model")
    model_sub = model_parser.add_subparsers(dest="model_command")

    pull_parser = model_sub.add_parser("pull")
    pull_parser.add_argument("--table", help="Pull apenas uma tabela")
    pull_parser.add_argument(
        "--force",
        action="store_true",
        help="Overscrew existing models"
    )

def handle_pull(args: list[str]):
    if not args:
        print("Error: No se proporcionaron argumentos.")
        sys.exit(1)
    
    action = args[0]          # pull
    rest = args[1:]           # users, --force
    table = None
    force = False
    for arg in rest:
        if arg == "--force":
            force = True
        else:
            table = arg
    if action == "pull":
        console.print(f"[info]Running model pull | table={table} | force={force}[/info]")
        
        project_root = get_project_root()

        if not project_root:
            console.print("[error]No Fleting projects found.[/error]")
            return
        
        db_path = find_database(project_root)
        if not db_path:
            console.print(
                "[error]❌ No local databases found.[/error]\n"
                "[suggestion]👉 Execute: fleting db init & fleting db migrate [/suggestion]"
            )
            return
        
        from core.table_filters import should_generate_model
        from configs.database import DATABASE
        engine = DATABASE.get("ENGINE", "sqlite").lower()
        conn = None
        cursor = None
        
        if engine == "sqlite":
            console.print("[success]🟢 SQLite detected[/success]")
            try:
                from core.database import get_connection
                import sqlite3

                conn = get_connection()
                conn.row_factory = sqlite3.Row
                cursor = conn.cursor()
            except Exception as e:
                console.print(f"[error]Error obtaining connection: {e}[/error]")
                return
        else:
            console.print(
                f"[error]❌ Engine '{engine}' not supported by the shell.[/error]"
            )
            return
        
        console.print("[info]🔍 Starting the introspection of the SQLite bank[/info]")

        excluded = DATABASE.get("MODEL_PULL", {}).get("EXCLUDE_TABLES", [])
        
        tables = [table] if table else get_tables(cursor)

        foreign_key_map = {}

        for tbl in tables:
            foreign_key_map[tbl] = get_foreign_keys(cursor, tbl)

        for tbl in tables:
            if not should_generate_model(tbl, excluded):
                console.print(f"[yellow]⏭ Skipping table {tbl}[/yellow]")
                continue
                    
            columns = get_columns(cursor, tbl)
            foreign_keys = foreign_key_map.get(tbl, [])

            if not columns:
                continue

            code = generate_model(
                tbl,
                columns,
                foreign_keys,
                foreign_key_map
            )

            save_model(project_root, tbl, code, force)

        console.print("[success]🎉 Models with relationships generated successfully[/success]")

    else:
        console.print( f"[error]Unknown model action: {action}[/error]\n"
            "[suggestion]Use: fleting db model pull[/suggestion]"
        )


    


from pathlib import Path
from fleting.cli.console.console import console

BASE = Path.cwd()

def handle_delete(args):
    if len(args) < 2:
        console.print("Uso: fleting delete <controller|view|model|page> <nome>", style="suggestion")
        return

    kind, name = args[0], args[1].lower()

    try:
        if kind == "view":
            delete_view(name)

        elif kind == "controller":
            delete_controller(name)

        elif kind == "model":
            delete_model(name)

        elif kind == "page":
            delete_page(name)

        else:
            console.print(f"Unsupported Type: {kind}", style="warning")

    except Exception:
        console.print(f"Error when deleting {kind} {name}", style="error")

# -----------------
# delete controller
# -----------------
def delete_controller(name: str):
    path = BASE / "controllers" / f"{name}_controller.py"

    if not path.exists():
        console.print(f"Controller '{name}' does not exist.", style="warning")
        return

    path.unlink()
    console.print(f"Controller successfully removed: {name}", style="success")

# -----------------
# delete view
# -----------------
def delete_view(name: str):
    path = BASE / "views" / "pages" / f"{name}_view.py"

    if not path.exists():
        console.print(f"View '{name}' does not exist.", style="warning")
        return

    path.unlink()
    console.print(f"View successfully removed: {name}", style="success")

# -----------------
# delete model
# -----------------
def delete_model(name: str):
    path = BASE / "models" / f"{name}_model.py"

    if not path.exists():
        console.print(f"Model '{name}' does not exist.", style="warning")
        return

    path.unlink()
    console.print(f"Model successfully removed: {name}", style="success")

# -----------------
# delete page
# -----------------
def delete_page(name: str):
    delete_view(name)
    delete_controller(name)
    delete_model(name)



from pathlib import Path
import sys

def is_fleting_project(path: Path) -> bool:
    return (path / ".fleting").exists()

def find_project_root(start=None) -> Path | None:
    if start is None:
        start = Path.cwd()
    elif isinstance(start, str):
        start = Path(start)

    start = start.resolve()

    if is_fleting_project(start):
        return start

    for parent in start.parents:
        if is_fleting_project(parent):
            return parent

    return None

def activate_project(root):
    from pathlib import Path
    import sys

    if isinstance(root, str):
        root = Path(root)

    root = root.resolve()

    if str(root) not in sys.path:
        sys.path.insert(0, str(root))

def get_project_root():
    root = find_project_root()
    if not root:
        print("❌ This directory is not a Fleting project.")
        print("👉 Go to the project root or a parent directory.")
        return

    activate_project(root)
    return root


from pathlib import Path
import shutil
import stat
import sys
from fleting.cli.templates.index import init_project
from fleting.cli.console.console import console

DEFAULT_PROJECT_NAME = "app"
DEFAULT_APP_NAME = "fleting"
CLI_NAME = "fleting" 

def _create_launcher(project_root: Path, cli_name: str = CLI_NAME):
    """Creates a launcher/shortcut for the Fleting executable"""
    BASE = project_root
    bin_dir = BASE / "bin"
    
    bin_dir.mkdir(parents=True, exist_ok=True)
    
    fleting_exe = None
    
    if sys.platform == "win32":
        # For Windows
        possible_names = ["fleting.exe", "fleting"]
    else:
        # For Unix-like (Linux, macOS)
        possible_names = ["fleting"]
    
    for name in possible_names:
        found = shutil.which(name)
        if found:
            fleting_exe = Path(found)
            break
    
    if not fleting_exe:
        console.print("Fleting not found in system", style="error")
        console.print("Project will be created without local launcher", style="warning")
        return False
    
    if sys.platform == "win32":
        # Windows: create .bat file named "fleting.bat"
        launcher_path = bin_dir / f"{cli_name}.bat"
        launcher_content = f'@"{fleting_exe}" %*'
        launcher_path.write_text(launcher_content, encoding='utf-8')
        
        # Optional: also create .ps1 for PowerShell
        ps_launcher = bin_dir / f"{cli_name}.ps1"
        ps_content = f'& "{fleting_exe}" @args'
        ps_launcher.write_text(ps_content, encoding='utf-8')
        
    else:
        # Unix-like: create shell script named "fleting"
        launcher_path = bin_dir / cli_name
        launcher_content = f"""#!/usr/bin/env bash
exec "{fleting_exe}" "$@"
"""
        launcher_path.write_text(launcher_content)
        
        try:
            launcher_path.chmod(launcher_path.stat().st_mode | 0o111)  # +x for all
        except:
            console.print(f"Could not make {launcher_path} executable", style="warning")
    
    return True

def _create_symlink(project_root: Path, cli_name: str = CLI_NAME):
    """Alternative: creates a direct symlink (Unix only)"""
    BASE = project_root
    bin_dir = BASE / "bin"
    
    bin_dir.mkdir(parents=True, exist_ok=True)
    
    fleting_exe = shutil.which(cli_name)
    if not fleting_exe:
        return False
    
    launcher_path = bin_dir / cli_name
    
    try:
        if sys.platform != "win32":
            # Unix: create symlink
            if launcher_path.exists():
                launcher_path.unlink()
            os.symlink(fleting_exe, launcher_path)
            return True
    except Exception as e:
        console.print(f"Error creating symlink: {e}", style="error")
    
    return False

def _bin(project_root: Path, cli_name: str = CLI_NAME):
    """Creates local binary/launcher - simplified version"""
    BASE = project_root
    bin_dir = BASE / "bin"
    
    bin_dir.mkdir(parents=True, exist_ok=True)
    
    if sys.platform != "win32":
        if _create_symlink(project_root, cli_name):
            console.print("Project created with local Fleting (symlink)", style="info")
            return
    
    if _create_launcher(project_root, cli_name):
        console.print("Project created with local Fleting (symlink)", style="info")
    # else:
    #     console.print("[warning]Project created without local Fleting access[/warning]")
    #     console.print("[info]Install Fleting globally or use full path[/info]")

def handle_init(args=None):
    args = args or []
    cwd = Path.cwd()

    if not args:
        project_root = cwd / DEFAULT_PROJECT_NAME
        project_name = DEFAULT_APP_NAME
        project_root.mkdir(parents=True, exist_ok=True)

    elif args[0] == ".":
        project_root = cwd
        project_name = DEFAULT_APP_NAME

    else:
        project_root = cwd / args[0]
        project_name = args[0].replace("_", " ").title()
        project_root.mkdir(parents=True, exist_ok=True)

    try:
        init_project(project_root, project_name)
        
        # Always use "fleting" as the CLI name for consistency
        _bin(project_root, CLI_NAME)
        
        console.print("✅ Project Fleting successfully initiated!", style="success")
        
         # Give instructions if Fleting wasn't found
        if not shutil.which("fleting"):
            console.print("\nNote: Fleting was not found in global PATH", style="info")
            console.print("To use the 'fleting' command locally, you need to:", style="info")
            console.print("  1. Install Fleting globally: pip install fleting", style="info")
            console.print("  2. Or use the full path to the executable", style="info")
        
        # Show usage instructions
        console.print("\nUsage:", style="info")
        console.print(f"  cd {project_root.name}")
        console.print("  bin\\fleting <command> # Access database shell (Windows)", style="suggestion")
        console.print("  bin/fleting <command>  # Access database shell (Unix)", style="suggestion")
            
    except Exception as e:
        console.print(f"Error creating project: {e}", style="error")
        raise


from pathlib import Path
from fleting.cli.console.console import console

def is_fleting_project(path: Path) -> bool:
    return (path / "main.py").exists() and (path / "views").exists()

def get_project_root() -> Path:
    return Path.cwd()

def handle_create(args):
    
    root = get_project_root()

    if not is_fleting_project(root):
        console.print("❌ This directory is not a Fleting project.", style="error")
        console.print("👉 Execute this command within the project folder.", style="suggestion")
        return
    
    if len(args) < 2:
        console.print("Use: fleting create <controller|view|model|page> <nome>", style="suggestion")
        return

    kind, name = args[0], args[1]
    name = name.lower()

    try:
        if kind == "controller":
            create_controller(name)
        elif kind == "view":
            create_view(name)
        elif kind == "model":
            create_model(name)
        elif kind == "page":
            create_page(name)
        else:
            console.print(f"Unsupported Type: {kind}", style="warning")

    except Exception as e:
        console.print(f"Error creating {kind} {name}: {e}", style="error")

# --------------
# create controller
# --------------
def to_pascal_case(text: str) -> str:
    return "".join(word.capitalize() for word in text.split("_"))

def create_controller(name: str):
    BASE = get_project_root()
    path = BASE / "controllers" / f"{name}_controller.py"

    if path.exists():
        console.print(f"Controller '{name}' already exists.", style="warning")
        return

    class_name = f"{to_pascal_case(name)}Controller"
    model_class = f"{to_pascal_case(name)}Model"

    content = f'''from models.{name}_model import {model_class}

class {class_name}:
    """
    Controller for {name} page
    """

    def __init__(self, model=None):
        self.model = model or {model_class}

    def get_title(self):
        return "{to_pascal_case(name)}"
'''
    path.write_text(content, encoding="utf-8")
    console.print(f"Controller successfully created: {name}", style="success")

# --------------
# create view
# --------------
def create_view(name: str):
    BASE = get_project_root()
    path = BASE / "views" / "pages" / f"{name}_view.py"

    if path.exists():
        console.print(f"View '{name}' already exists.", style="warning")
        return

    class_name = f"{name.capitalize()}View"

    content = f"""import flet as ft
from views.layouts.main_layout import MainLayout

class {class_name}:
    def __init__(self, page, router):
        self.page = page
        self.router = router

    def render(self):
        content = ft.Column(
            controls=[
                ft.Text("{name.capitalize()}", size=24),
            ],
            spacing=16,
        )

        return MainLayout(
            page=self.page,
            content=content,
            router=self.router,
        )
"""

    path.write_text(content, encoding="utf-8")
    console.print(f"View successfully created: {name}", style="success")


# --------------
# create model
# --------------
def create_model(name: str):
    BASE = get_project_root()
    path = BASE / "models" / f"{name}_model.py"

    if path.exists():
        console.print(f"Model '{name}' already exists.", style="warning")
        return

    class_name = f"{name.capitalize()}Model"

    content = f"""from core.base_model import BaseModel

class {class_name}(BaseModel):
    table_name = "{name}"
"""

    path.write_text(content, encoding="utf-8")
    console.print(f"Model successfully created: {name}", style="success")

# --------------
# create page
# --------------
def create_page(name: str):
    console.print(f"Creating a complete page: {name}", style="info")
    try:
        create_model(name)
        create_controller(name)
        create_page_view(name)
        register_route(name)
    except Exception as e:
        console.print("Error creating page: ", str(e), style="error")

def register_route(name: str):
    BASE = get_project_root()
    routes_file = BASE / "configs" / "routes.py"

    if not routes_file.exists():
        console.print("❌ configs/routes.py não not found", style="error")
        return

    route_block = f"""
    {{
        "path": "/{name}",
        "view": "views.pages.{name}_view.{name.capitalize()}View",
        "label": "{name.capitalize()}",
        "icon": ft.Icons.CHEVRON_RIGHT,
        "show_in_top": True,
        "show_in_bottom": False,
    }},
"""

    content = routes_file.read_text(encoding="utf-8")

    # evita duplicar
    if f'"path": "/{name}"' in content:
        console.print(f"⚠️ Route '/{name}' already exists.", style="warning")
        return

    if "ROUTES = [" not in content:
        console.print("❌ ROUTES structure not found", style="error")
        return

    content = content.replace(
        "ROUTES = [",
        "ROUTES = [" + route_block,
        1,
    )

    routes_file.write_text(content, encoding="utf-8")
    console.print(f"✅ Route '/{name}' successfully registered", style="success")

def create_page_view(name: str):
    BASE = get_project_root()
    path = BASE / "views" / "pages" / f"{name}_view.py"

    if path.exists():
        console.print(f"View '{name}' already exists.", style="warning")
        return

    class_name = f"{name.capitalize()}View"
    controller_class = f"{name.capitalize()}Controller"
    model_class = f"{name.capitalize()}Model"

    content = f"""import flet as ft
from views.layouts.main_layout import MainLayout
from controllers.{name}_controller import {controller_class}

class {class_name}:
    def __init__(self, page, router):
        self.page = page
        self.router = router
        self.controller = {controller_class}()

    def render(self):
        content = ft.Column(
            controls=[
                ft.Text(self.controller.get_title(), size=24),
            ],
            spacing=16,
        )

        return MainLayout(
            page=self.page,
            content=content,
            router=self.router,
        )
"""
    path.write_text(content, encoding="utf-8")
    console.print(f"Page created successfully: {name}", style="success")



from pathlib import Path
import json
import base64

ICON = "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"

def init_project(project_root: Path, project_name: str = "Fleting"):
    """
    Initializes a Fleting project in the current directory.
    """

    BASE = project_root

    # =========================
    # UTIL
    # =========================
    def create_file(path, content=""):
        path.parent.mkdir(parents=True, exist_ok=True)
        if not path.exists():
            path.write_text(content, encoding="utf-8")

    # =========================
    # ESTRUTURA DE PASTAS
    # =========================
    folders = [
        "assets",
        "core",
        "configs/languages",
        "controllers",
        "models",
        "views/layouts",
        "views/pages",
    ]

    for folder in folders:
        (BASE / folder).mkdir(parents=True, exist_ok=True)
    
    

    # =========================
    # ICON FLETING
    # =========================
    try:
        with open(BASE / "assets/icon.png", "wb") as f:
            f.write(base64.b64decode(ICON))
    except Exception as e:
        print(f"❌ Error creating icon: {str(e)}")
        pass
    # =========================
    # ARQUIVOS CORE
    # =========================
    create_file(BASE / "core/state.py", """
class AppState:
    device = None  # mobile | tablet | desktop
    initial_device = "mobile"
    language = "pt"
    initialized = False
    current_route = "/"
""")

    create_file(BASE / "core/responsive.py", """
def get_device_type(width):
    if width <= 600:
        return "mobile"
    elif width <= 1024:
        return "tablet"
    return "desktop"
""")

    create_file(BASE / "core/router.py", """
import flet as ft
from core.logger import get_logger

logger = get_logger("Router")
class Router:
    def __init__(self, page):
        self.page = page
        self.current_route = "/"
        self.routes = self._load_routes()

    def _load_routes(self):
        from configs.routes import routes
        return routes

    def navigate(self, route):
        routes = self.routes

        if route not in routes:
            logger.warning(f"Route not found: {route}")
            route = "/"

        logger.info(f"Navigating to: {route}")
        self.current_route = route
        self.page.controls.clear()

        try:
            view = routes[route](self.page, self)
            self.page.add(view)
        except Exception as e:
            logger.exception("Error rendering view")
            self.page.add(ft.Text("Internal application error"))

        self.page.update()
""")

    create_file(BASE / "core/app.py", """
import flet as ft
from core.responsive import get_device_type
from core.state import AppState
from core.i18n import I18n

class FletingApp:
    def __init__(self, page):
        self.page = page
        AppState.device = AppState.initial_device
        self.page.on_resize = self.on_resize
        I18n.load(AppState.language)
        self.page.appbar = self.build_topbar()
        from core.router import Router
        self.router = Router(page)
        self.router.navigate("/")
    
    def build_topbar(self):
        menu_items = []

        menu = I18n.translations.get("menu", {})

        for route, label in menu.items():
            menu_items.append(
                ft.TextButton(
                    text=label,
                    icon=ft.icons.CIRCLE,
                    on_click=lambda e, r=f"/{route}": self.router.navigate(r),
                )
            )

        return ft.AppBar(
            title=ft.Text(I18n.t("app.name")),
            actions=menu_items,
            center_title=False,
        )

    def on_resize(self, e):
        real_device = get_device_type(self.page.width)

        # Avoid overwriting on the first fake frame
        if not AppState.initialized:
            AppState.initialized = True

        AppState.device = real_device
        self.page.update()
""")

    create_file(BASE / "core/i18n.py", """
import json
from pathlib import Path
from core.state import AppState

class I18n:
    translations = {}

    @classmethod
    def load(cls, lang):
        current_file = Path(__file__).resolve()
        base_path = current_file.parent.parent
        path = base_path / "configs" / "languages" / f"{lang}.json"
        cls.translations = json.loads(path.read_text(encoding="utf-8"))
        AppState.language = lang

    @classmethod
    def t(cls, key):
        value = cls.translations
        for k in key.split("."):
            value = value.get(k)
            if value is None:
                return key
        return value
""")

    create_file(BASE / "core/logger.py", """
import logging
import sys
import os
from pathlib import Path

APP_NAME = "fleting"

def is_frozen():
    return getattr(sys, "frozen", False)

def is_android():
    return sys.platform == "android"

def get_log_dir():
    # ANDROID (APK)
    if is_android():
        return Path(os.getcwd()) / "files" / "logs"

    # EXECUTABLE (PyInstaller)
    if is_frozen():
        base = Path(os.getenv("LOCALAPPDATA", Path.home()))
        return base / APP_NAME / "logs"

    # DESENVOLVIMENTO
    return Path.cwd() / "logs"

LOG_DIR = get_log_dir()
LOG_DIR.mkdir(parents=True, exist_ok=True)

LOG_FILE = LOG_DIR / "fleting.log"

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s | %(levelname)s | %(name)s | %(message)s",
    handlers=[
        logging.FileHandler(LOG_FILE, encoding="utf-8"),
        logging.StreamHandler(),
    ],
)

def get_logger(name: str):
    return logging.getLogger(name)
""")

    create_file(BASE / "core/error_handler.py", """
import flet as ft
from core.logger import get_logger

logger = get_logger("ErrorHandler")

class GlobalErrorHandler:
    @staticmethod
    def handle(page: ft.Page, error: Exception):
        logger.exception("Global error caught")

        page.controls.clear()
        page.add(
            ft.Container(
                content=ft.Column(
                    controls=[
                        ft.Text("⚠️ An error occurred.", size=24, weight=ft.FontWeight.BOLD),
                        ft.Text("Something went wrong. Please try again."),
                    ],
                    alignment=ft.MainAxisAlignment.CENTER,
                    horizontal_alignment=ft.CrossAxisAlignment.CENTER,
                ),
                expand=True,
            )
        )
        page.update()
""")

    create_file(BASE / "core/database.py", """

import sqlite3
# import mysql.connector  
from configs.database import DATABASE
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent
_connection = None

def get_connection():
    global _connection

    if _connection is not None:
        return _connection

    engine = DATABASE.get("ENGINE", "sqlite").lower()

    if engine == "sqlite":
        _connection = _connect_sqlite()
    elif engine == "mysql":
        _connection = _connect_mysql()
    else:
        raise RuntimeError(f"Unsupported database engine: {engine}")

    return _connection

# =========================
# SQLITE
# =========================
def _connect_sqlite():
    cfg = DATABASE.get("SQLITE", {})
    path_cfg = cfg.get("PATH", "data/fleting.db")

    db_path = BASE_DIR / Path(path_cfg)
    db_path.parent.mkdir(parents=True, exist_ok=True)

    return sqlite3.connect(db_path)

# =========================
# MYSQL
# =========================
def _connect_mysql():
    pass
    # try:
    #     import mysql.connector
    # except ImportError:
    #     raise RuntimeError(
    #         "MySQL support requires `mysql-connector-python` :"
    #         "pip install mysql-connector-python"
    #     )

    # cfg = DATABASE.get("MYSQL", {})

    # return mysql.connector.connect(
    #     host=cfg.get("HOST", "localhost"),
    #     port=cfg.get("PORT", 3306),
    #     user=cfg.get("USER"),
    #     password=cfg.get("PASSWORD"),
    #     database=cfg.get("NAME"),
    #     charset=cfg.get("OPTIONS", {}).get("charset", "utf8mb4"),
    # )

""")
    
    create_file(BASE / "core/base_model.py", """
# core/base_model.py
from core.database import get_connection
from core.orm import ForeignKey, HasMany
from typing import Any

class QuerySet:

    def __init__(self, model, relations=None):
        self.model = model
        self.relations = relations or []
    
    def _cursor(self):
        conn = get_connection()
        conn.row_factory = lambda c, r: dict(
            zip([col[0] for col in c.description], r)
        )
        return conn, conn.cursor()

    def all(self):
        conn, cursor = self._cursor()

        cursor.execute(
            f"SELECT * FROM {self.model.table_name}"
        )
        rows = cursor.fetchall()

        return self._attach_relations(cursor, rows)
    
    def first(self):
        conn, cursor = self._cursor()
        cursor.execute(
            f"SELECT * FROM {self.model.table_name} LIMIT 1"
        )
        row = cursor.fetchone()
        if not row:
            return None
        return self._attach_relations(cursor, [row])[0]

    def find(self, id):
        conn, cursor = self._cursor()

        cursor.execute(
            f"SELECT * FROM {self.model.table_name} WHERE id = ?",
            (id,)
        )
        row = cursor.fetchone()

        if not row:
            return None

        return self._attach_relations(cursor, [row])[0]

    def where(self, **filters):
        conn, cursor = self._cursor()

        keys = filters.keys()
        values = tuple(filters.values())
        cond = " AND ".join(f"{k}=?" for k in keys)

        cursor.execute(
            f"SELECT * FROM {self.model.table_name} WHERE {cond}",
            values
        )

        rows = cursor.fetchall()
        return self._attach_relations(cursor, rows)
    
    def count(self):
        conn, cursor = self._cursor()
        cursor.execute(
            f"SELECT COUNT(*) as total FROM {self.model.table_name}"
        )
        return cursor.fetchone()["total"]
    
    def exists(self, **filters):
        return len(self.where(**filters)) > 0
    
    # =========================
    # Relations
    # =========================

    def _attach_relations(self, cursor, rows):
        if not self.relations:
            return rows

        for row in rows:
            for rel_name in self.relations:
                rel = getattr(self.model, rel_name, None)

                if isinstance(rel, ForeignKey):
                    value = row.get(rel.local)
                    row[rel_name] = rel.resolve(cursor, value)

                elif isinstance(rel, HasMany):
                    value = row.get("id")
                    row[rel_name] = rel.resolve(cursor, value)

        return rows

class BaseModel:
    table_name: str
    # ---------- Query ----------
    @classmethod
    def with_(cls, *relations):
        return QuerySet(cls, relations)

    @classmethod
    def all(cls):
        return cls.with_().all()
    
    @classmethod
    def first(cls):
        return cls.with_().first()

    @classmethod
    def find(cls, id):
        return cls.with_().find(id)

    @classmethod
    def where(cls, **filters):
        return cls.with_().where(**filters)
    
    @classmethod
    def count(cls):
        return cls.with_().count()
    
    @classmethod
    def exists(cls, **filters):
        return cls.with_().exists(**filters)

    # ---------- Write ----------
    @classmethod
    def create(cls, **data):
        conn = get_connection()
        cursor = conn.cursor()

        keys = ", ".join(data.keys())
        placeholders = ", ".join("?" for _ in data)
        values = tuple(data.values())

        cursor.execute(
            f"INSERT INTO {cls.table_name} ({keys}) VALUES ({placeholders})",
            values
        )
        conn.commit()

        return cls.find(cursor.lastrowid)

    def to_dict(self):
        return {
            k: v
            for k, v in self.__dict__.items()
            if not k.startswith("_")
        }

    def save(self):
        data = self.__dict__.copy()

        if "id" in data and data["id"]:
            self.update(**data)
        else:
            obj = self.__class__.create(**data)
            self.id = obj["id"]

        return self

    def update(self, **data):
        conn = get_connection()
        cursor = conn.cursor()

        if "id" not in data:
            raise ValueError("Update requires id")

        id = data.pop("id")
        assigns = ", ".join(f"{k}=?" for k in data)
        values = tuple(data.values()) + (id,)

        cursor.execute(
            f"UPDATE {self.table_name} SET {assigns} WHERE id = ?",
            values
        )
        conn.commit()
        return self

    def delete(self):
        if not getattr(self, "id", None):
            raise ValueError("Delete requires id")

        conn = get_connection()
        cursor = conn.cursor()

        cursor.execute(
            f"DELETE FROM {self.table_name} WHERE id = ?",
            (self.id,)
        )
        conn.commit()
 """)

    create_file(BASE / "core/orm.py", """
# core/orm.py
class ForeignKey:
    def __init__(self, table: str, local: str, remote: str = "id"):
        self.table = table
        self.local = local
        self.remote = remote

    def resolve(self, cursor, value):
        cursor.execute(
            f"SELECT * FROM {self.table} WHERE {self.remote} = ?",
            (value,)
        )
        row = cursor.fetchone()
        return dict(row) if row else None

    def __repr__(self):
        return f"<ForeignKey {self.local} → {self.table}.{self.remote}>"

class HasMany:
    def __init__(self, table: str, foreign_key: str):
        self.table = table
        self.foreign_key = foreign_key

    def resolve(self, cursor, value):
        cursor.execute(
            f"SELECT * FROM {self.table} WHERE {self.foreign_key} = ?",
            (value,)
        )
        return [dict(r) for r in cursor.fetchall()]

    def __repr__(self):
        return f"<HasMany {self.table} via {self.foreign_key}>"

 """)
    
    create_file(BASE / "core/table_filters.py", """
DEFAULT_EXCLUDED_TABLES = {
    "_fleting_migrations",
    "migrations",
    "schema_migrations",
    "alembic_version",
    "sqlite_sequence",
    "flyway_schema_history",
}

def is_internal_table(table_name: str) -> bool:

    return (
        table_name.startswith("_")
        or table_name.startswith("sys_")
        or table_name.startswith("tmp_")
    )

def should_generate_model(table_name: str, config_excludes=None) -> bool:

    config_excludes = set(config_excludes or [])

    if table_name in DEFAULT_EXCLUDED_TABLES:
        return False

    if table_name in config_excludes:
        return False

    if is_internal_table(table_name):
        return False

    return True
 """)
   
    # =========================
    # CONFIGS
    # =========================
    create_file(BASE / "configs/app_config.py", """
# configs/app_config.py
import flet as ft

class ScreenConfig:
    MOBILE = {
        "width": 390,
        "height": 758,
        "max_content_width": 390,
    }

    TABLET = {
        "width": 768,
        "height": 1024,
        "max_content_width": 768,
    }

    DESKTOP = {
        "width": 1280,
        "height": 800,
        "max_content_width": None,  # no limit
    }

class AppConfig:
    APP_NAME = "{project_name}"
    DEFAULT_SCREEN = ScreenConfig.MOBILE
    THEME_MODE = ft.ThemeMode.LIGHT
""")

    create_file(BASE / "configs/routes.py", """
import flet as ft
import importlib

ROUTES = [
    {
        "path": "/",
        "view": "views.pages.home_view.HomeView",
        "label": "menu.home",
        "icon": ft.Icons.HOME,
        "show_in_top": True,
        "show_in_bottom": True,
    },
    {
        "path": "/settings",
        "view": "views.pages.settings_view.SettingsView",
        "label": "menu.settings",
        "icon": ft.Icons.SETTINGS,
        "show_in_top": True,
        "show_in_bottom": False,
    },
    {
        "path": "/help",
        "view": "views.pages.help_view.HelpView",
        "label": "menu.help",
        "icon": ft.Icons.HELP,
        "show_in_top": True,
        "show_in_bottom": True,
    }
]

def load_view(view_path: str):
    module_name, class_name = view_path.rsplit(".", 1)
    
    try:
        module = importlib.import_module(module_name)
        view_class = getattr(module, class_name)
        return view_class
    except (ImportError, AttributeError) as e:
        print(f"Erro ao carregar view {view_path}: {e}")
        return None

def get_routes():
    routes = {}

    for r in ROUTES:
        def create_view_lambda(path=r["view"]):
            return lambda page, router: load_view(path)(page, router).render()

        routes[r["path"]] = create_view_lambda()

    return routes

routes = get_routes()

""")
    
    create_file(BASE / "configs/database.py", """
DATABASE = {
    "ENGINE": "sqlite",  # sqlite | mysql

    "SQLITE": {
        "PATH": "data/fleting.db"
    },

    "MYSQL": {
        "HOST": "localhost",
        "PORT": 3306,
        "USER": "root",
        "PASSWORD": "",
        "NAME": "fleting",
        "OPTIONS": {
            "charset": "utf8mb4"
        }
    }
}
""")

    # =========================
    # LANGUAGES
    # =========================
    en = {
    "app": {
        "name": project_name
    },
    "menu": {
        "home": "Home",
        "settings": "Configs",
        "help": "Help"
    },
    "home": {
        "title": f"Wellcome to {project_name}"
    },
    "settings": {
        "title": "Configs",
        "language": "Language"
    }
    }

    pt = {
    "app": {
        "name": project_name
    },
    "menu": {
        "home": "Inicio",
        "settings": "Configurações",
        "help": "Ajuda"
    },
    "home": {
        "title": f"Bem-vindo ao {project_name}"
    },
    "settings": {
        "title": "Configurações",
        "language": "Idioma"
    }
    }

    es = {
    "app": {
        "name": project_name
    },
    "menu": {
        "home": "Inicio",
        "settings": "Configuración",
        "help": "Ayuda"
    },
    "home": {
        "title": f"Bienvenido a {project_name}"
    },
    "settings": {
        "title": "Configuración",
        "language": "Idioma"
    }
    }

    pt_file = f"{BASE}/configs/languages/pt.json"
    es_file = f"{BASE}/configs/languages/es.json"
    us_file = f"{BASE}/configs/languages/en.json"
    with open(pt_file, 'w', encoding='utf-8') as f:
        json.dump(pt, f, indent=2, ensure_ascii=False)
    
    with open(es_file, 'w', encoding='utf-8') as f:
        json.dump(es, f, indent=2, ensure_ascii=False)

    with open(us_file, 'w', encoding='utf-8') as f:
        json.dump(en, f, indent=2, ensure_ascii=False)

    # =========================
    # LAYOUT
    # =========================
    create_file(BASE / "views/layouts/main_layout.py", """
import flet as ft
from core.state import AppState
from core.i18n import I18n
from configs.routes import ROUTES

class MainLayout(ft.Column):
    def __init__(self, page, content, router):
        super().__init__(expand=True)
        self._page = page
        self.router = router
        self.content = content

        self._build()

    def _build(self):
        self.controls.clear()

        # TOP BAR
        self.controls.append(self._top_bar())

        # CONTENT
        self.controls.append(
            ft.Container(
                content=self.content,
                expand=True,
                padding=0,
            )
        )

        # BOTTOM BAR (mobile / tablet)
        if AppState.device != "desktop":
            self.controls.append(self._bottom_bar())

    # ---------- TOP BAR ----------
    def _top_bar(self):
        items = []

        for r in ROUTES:
            if not r.get("show_in_top"):
                continue

            items.append(
                ft.PopupMenuItem(
                    content=ft.Row(
                        controls=[
                            ft.Icon(r["icon"]),
                            ft.Text(
                                I18n.t(r["label"]) if "." in r["label"] else r["label"]
                            ),
                        ],
                        spacing=10,
                    ),
                    on_click=lambda e, p=r["path"]: self.router.navigate(p),
                )
            )

        return ft.AppBar(
            title=ft.Text(I18n.t("app.name")),
            actions=[
                ft.PopupMenuButton(
                    icon=ft.Icons.MENU,
                    items=items,
                )
            ],
        )

    # ---------- BOTTOM BAR ----------
    def _bottom_bar(self):
        destinations = []
        paths = []

        for r in ROUTES:
            if r.get("show_in_bottom"):
                destinations.append(
                    ft.NavigationBarDestination(
                        icon=r["icon"],
                        label=I18n.t(r["label"]),
                    )
                )
                paths.append(r["path"])

        def on_change(e):
            self.router.navigate(paths[e.control.selected_index])

        return ft.NavigationBar(
            destinations=destinations,
            selected_index=paths.index(AppState.current_route)
            if AppState.current_route in paths else 0,
            on_change=on_change,
        )
""")

    # =========================
    # VIEW HOME
    # =========================
    create_file(BASE / "views/pages/home_view.py", """
import flet as ft
from views.layouts.main_layout import MainLayout

class HomeView:
    def __init__(self, page, router):
        self.page = page
        self.router = router
    
    def render(self):
        content = ft.Column(
            alignment=ft.MainAxisAlignment.CENTER,
            horizontal_alignment=ft.CrossAxisAlignment.CENTER,
            spacing=24,
            controls=[
                ft.Image(
                    src="icon.png",
                    width=96,
                    height=96,
                    fit="contain",
                ),
                ft.Text(
                    "Fleting Framework",
                    size=36,
                    weight=ft.FontWeight.BOLD,
                ),
                ft.Text(
                    "Micro Framework MVC for Flet",
                    size=16,
                    color=ft.Colors.GREY_600,
                ),
                ft.Text(
                    "Build modern applications with a clear architecture, "
                    "Dynamic routing and productive CLI.",
                    size=14,
                    text_align=ft.TextAlign.CENTER,
                    width=420,
                ),

                ft.Row(
                    alignment=ft.MainAxisAlignment.CENTER,
                    spacing=16,
                    controls=[
                        ft.FilledButton(
                            "Settings",
                            icon=ft.Icons.SETTINGS,
                            on_click=lambda e: self.router.navigate("/settings"),
                        ),
                        ft.OutlinedButton(
                            "Create new page",
                            icon=ft.Icons.ADD,
                        ),
                    ],
                ),
            ],
        )

        # LAYOUT
        return MainLayout(
            page=self.page,
            content=content,
            router=self.router,
        )
""")

    create_file(BASE / "views/pages/settings_view.py", """
import flet as ft
from core.state import AppState
from core.i18n import I18n
from views.layouts.main_layout import MainLayout
from controllers.settings_controller import SettingsController

class SettingsView:
    def __init__(self, page, router):
        self.page = page
        self.router = router
        self.controller = SettingsController()
    
    def _change_language(self, lang: str):
        I18n.load(lang)
        self.page.update()

    def render(self):
        content = ft.Column(
            spacing=24,
            controls=[
                ft.Text(
                    self.controller.get_title(),
                    size=28,
                    weight=ft.FontWeight.BOLD,
                ),

                ft.Text(
                    I18n.t("settings.language"),
                    size=16,
                    color=ft.Colors.GREY_600,
                ),

                ft.RadioGroup(
                    value=AppState.language,
                    on_change=lambda e: self._change_language(e.control.value),
                    content=ft.Column(
                        controls=[
                            ft.Radio(value="pt", label="Português 🇧🇷"),
                            ft.Radio(value="en", label="English 🇺🇸"),
                            ft.Radio(value="es", label="Español 🇪🇸"),
                        ]
                    ),
                ),
            ],
        )

        return MainLayout(
            page=self.page,
            content=content,
            router=self.router,
        )
""")

    create_file(BASE / "views/pages/help_view.py", """
import flet as ft
from views.layouts.main_layout import MainLayout
from controllers.help_controller import HelpController
from flet import UrlLauncher

class HelpView:
    def __init__(self, page, router):
        self.page = page
        self.router = router
        self.controller = HelpController()
        self.url_launcher = UrlLauncher()
    
    async def open_docs(self, e):
        await self.url_launcher.launch_url("https://github.com/alexyucra/Fleting")

    async def open_issues(self, e):
        await self.url_launcher.launch_url("https://github.com/alexyucra/fleting/issues")

    async def open_support(self, e):
        await self.url_launcher.launch_url("https://alexyucra.github.io/#contato")

    def render(self):
        

        content = ft.Column(
            spacing=24,
            controls=[
                ft.Text(
                    self.controller.get_title(),
                    size=28,
                    weight=ft.FontWeight.BOLD,
                ),

                ft.Text(
                    "Precisa de ajuda com o Fleting?",
                    size=18,
                    weight=ft.FontWeight.W_500,
                ),

                ft.Text(
                    "Aqui você encontra links úteis para documentação, "
                    "suporte e contribuição com o projeto.",
                    color=ft.Colors.GREY_600,
                ),

                ft.Divider(),

                ft.Button(
                    "📘 Documentação Oficial",
                    icon=ft.Icons.MENU_BOOK,
                    on_click=self.open_docs,
                ),

                ft.Button(
                    "🐛 Reportar um problema",
                    icon=ft.Icons.BUG_REPORT,
                    on_click=self.open_issues,
                ),

                ft.Button(
                    "💬 Precisa de uma automação ou Sistema?",
                    icon=ft.Icons.BUG_REPORT,
                    on_click=self.open_support,
                ),
            ],
        )

        return MainLayout(
            page=self.page,
            content=content,
            router=self.router,
        )
""")

    # =========================
    # APP ENTRY
    # =========================
    (BASE / ".fleting").write_text("fleting-project", encoding="utf-8")

    create_file(BASE / "main.py", """
import flet as ft
from configs.app_config import AppConfig
from core.logger import get_logger
from core.error_handler import GlobalErrorHandler
import runtime_imports 

logger = get_logger("App")

def main(page: ft.Page):
    try:
        page.assets_dir = "assets"

        if page.platform in (ft.PagePlatform.WINDOWS, ft.PagePlatform.LINUX, ft.PagePlatform.MACOS):
            from core.app import FletingApp
            page.window.width = AppConfig.DEFAULT_SCREEN["width"]
            page.window.height = AppConfig.DEFAULT_SCREEN["height"]

        from core.i18n import I18n
        I18n.load("pt")

        from core.router import Router
        from configs.routes import routes

        router = Router(page)
        router.navigate("/")

        logger.info("Aplicação iniciada com sucesso")
        
    except Exception as e:
        GlobalErrorHandler.handle(page, e)

ft.app(main)

""")
    create_file(BASE / "runtime_imports.py", """
from views.pages.home_view import HomeView
from views.pages.settings_view import SettingsView
from views.pages.help_view import HelpView
""")

    # =========================
    # BASIC CONTROLLERS
    # =========================
    create_file(BASE / "controllers/settings_controller.py", """
from models.settings_model import SettingsModel

class SettingsController:
    '''
    Controller for settings page
    '''

    def __init__(self, model=None):
        self.model = model or SettingsModel

    def get_title(self):
        return "Settings"
""")

    create_file(BASE / "controllers/help_controller.py", """
from models.help_model import HelpModel

class HelpController:
    '''
    Controller for help page
    '''

    def __init__(self, model=None):
        self.model = model or HelpModel

    def get_title(self):
        return "Help"
""")

    # =========================
    # BASIC models
    # =========================
    create_file(BASE / "models/help_model.py", """
from core.base_model import BaseModel

class HelpModel(BaseModel):
    table_name = "help"
""")

    create_file(BASE / "models/settings_model.py", """
from core.base_model import BaseModel

class SettingsModel(BaseModel):
    table_name = "settings"
""")

if __name__ == "__main__":
    init_project()


# cli/console/console.py
from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich.columns import Columns
from rich.text import Text
from rich import box
from rich.rule import Rule
from rich.syntax import Syntax
from rich.markdown import Markdown
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import Prompt, Confirm
from rich.tree import Tree
from fleting.cli.console.themes import THEMES

console = Console(theme=THEMES)

def create_table(title: str = "", columns: list = None, min_width: int = 30) -> Table:
    """Create a table with Fleting stile"""
    table = Table(
        show_header=True,
        header_style="bold cyan",
        box=box.ROUNDED,
        border_style="blue",
        padding=(0, 1),
        min_width=min_width,
        expand=True
    )
    
    if title:
        table.title = f"[bold yellow]{title}[/bold yellow]"
    
    if columns:
        for col in columns:
            if isinstance(col, tuple):
                table.add_column(col[0], style=col[1], width=col[2] if len(col) > 2 else None)
            else:
                table.add_column(col, style="cyan")
    
    return table

def print_simple_table(headers: list, rows: list, title: str = ""):
    """Print a simple table with styled rows"""
    table = create_table(title, headers)
    
    for row in rows:
        styled_row = []
        for cell in row:
            cell_str = str(cell)
            # Apply styles based on content
            if cell_str in ["✔", "✅", "✓"]:
                styled_cell = "[green]✔[/green]"
            elif cell_str in ["—", "✗", "❌"]:
                styled_cell = "[dim]—[/dim]"
            elif "bytes" in cell_str.lower():
                styled_cell = f"[dim]{cell_str}[/dim]"
            else:
                styled_cell = f"[cyan]{cell_str}[/cyan]"
            styled_row.append(styled_cell)
        
        table.add_row(*styled_row)
    
    console.print()
    console.print(table)
    console.print()
    
def print_pages_table(headers: list, rows: list):
    """print page tables with specific colors"""
    table = create_table("📄 Pages Overview", headers, min_width=50)
    
    for row in rows:
        styled_row = []
        for i, cell in enumerate(row):
            if i == 0:  # page name
                styled_cell = f"[bold cyan]{cell}[/bold cyan]"
            elif cell == "✔":
                styled_cell = "[green]✔[/green]"
            elif cell == "—":
                styled_cell = "[dim red]—[/dim red]"
            else:
                styled_cell = f"[yellow]{cell}[/yellow]"
            styled_row.append(styled_cell)
        
        table.add_row(*styled_row)
    
    console.print()
    console.print(table)
    console.print(f"[dim]Legend: [green]✔[/green] = Present | [dim red]—[/dim red] = Missing[/dim]")
    console.print()

def print_routes_table(headers: list, rows: list):
    """Print routes table with especial style"""
    table = create_table("🛣️ Routes", headers, min_width=40)
    
    for row in rows:
        styled_row = []
        styled_row.append(f"[bold green]{row[0]}[/bold green]")  # Route
        styled_row.append(f"[cyan]{row[1]}[/cyan]")  # View
        table.add_row(*styled_row)
    
    console.print()
    console.print(table)

__all__ = [
    'console',
    'Console',
    'Table',
    'Panel',
    'Columns',
    'Text',
    'box',
    'Rule',
    'Syntax',
    'Markdown',
    'Progress',
    'SpinnerColumn',
    'TextColumn',
    'Prompt',
    'Confirm',
    'Tree',
    'create_table',
    'print_simple_table',
    'print_pages_table',
    'print_routes_table',
]

